% Example use of the code for Eli
% April 11 Taeg Sang Cho
clear;

nIter = 10;  % Number of iterations
maxProductFlag = 0; % 1 if we want to compute the max-product version of BP
alpha = 0.5; % Message update rate: typically 0.5
exclusive = 1; % Exclusivity constraint flag
sigEvid =0.25; % Sigma for local evidence 
sigExcl = 1;% Sigma for exclusivity
withEvidence = 1; % Flag for using local evidence for BP computation
blend = 0; % 
randInit =1; % Randomize the initial condition for BP
noPatches = 25;
K_states = noPatches; % Number of states for each node in MRF

M_Nodes = 5; % # of rows in the image
N_Nodes = 5; % # of columns in the image

% no patches to move around...
% these can have numbers between 1 and 25
patchMoveFrom = []; 
patchMoveTo = [];
patchFix = [];

% making the compatibility matrix.
% compLR(i, j) is the compatibility of placing patch i to the right of j
compLRT = min(max(circshift(eye(25) + 0.1*randn(25), [0, -1]), 0), 1);
compLRT(1, 25) = 0;
compLR = compLRT;

% compDU(i, j) is the compatibility of placing patch i on top of j
compDUT = min(max(circshift(eye(25) + 0.1*randn(25), [0, N_Nodes]), 0), 1);
compDUT(25 - N_Nodes + 1 : 25, :) = 0.1*randn(N_Nodes, 25);
compDU = min(max(compDUT, 0), 1);

% making the patches and the evidence
% patch is a M_Nodes x N_Nodes x dim x noPatches size variable where dim is
% 1 if the patches are gray scale and 3 if color.
num = reshape([1:25], [5, 5])';
patch = reshape([1:25], [1, 1, 1, 25]);
% evidence is a M_Nodes x N_Nodes x dim size variable where dim is
% 1 if the patches are gray scale and 3 if color.
evidence = reshape(num(:), [5, 5]) + 0.8*randn([5,5]) ;

% MRF Initialization + compatibility initialization
% nodes are enumerated in a horizontal-first scan fashion.
[nodes] = initMRFPatchWConst(M_Nodes, N_Nodes, K_states, compDU, compLR);

% Local evidence initialization
[nodes, evidMatrix] = initPatchToNodeWFixVar(nodes, patch, evidence, noPatches, sigEvid, patchMoveFrom,...
    patchMoveTo, patchFix, withEvidence);

% MRF node message initialization
[nodes] = initBPMessagesWConst(nodes, K_states, exclusive, randInit);

% Running the BP
tic;
for i =1:nIter
    nodes = oneIterBPDampWConstMod(nodes, maxProductFlag, alpha, K_states, exclusive, sigExcl);
    i
end    
nodes = computeBeliefsWConst(nodes, exclusive);
toc;

% Showing the evidence matrix
figure, imshow(evidMatrix)
title('Initial evidence')

% Showing the belief matrix
beliefMat = zeros(noPatches, noPatches);
for i = 1:noPatches
    beliefMat(i, :) = nodes{i}.marginal;
end
figure, imshow(beliefMat)
title('belief after running BP')