% initMRF.m
% modified from Egon's rmNodesFromCoeffs.m
% input:  the matrix showing which numbered image segments are
%         connected to which other nodes (neighborMat).  
%         The likelihood for each segment for each disparity (lhoods).
% output: nodes structure, ready to send into propNodes
%
% Modified from Bill's initMRF.m to also allow multiple (K-states) states.
% Bill's programs can only run for binary states
% Taeg Sang Cho, Oct 1 2007
% Now, able to initialize MRFs with a global constraint
% Taeg Sang Cho, Oct 23 2007
%
% Trimmed for Eli
% Taeg Sang Cho, April 10 2008

function [nodes] = initMRFPatchWConst(M_Nodes, N_Nodes, K_states, compDU, compLR)

fprintf(1, 'Entering initMRFPatchWConst \n'); tic;                    %%

nNodes = N_Nodes * M_Nodes;
warning off all;

cDU = compDU;
cUD = compDU';
cLR = compLR;
cRL= compLR';

%% Generating nodes
nodes = cell(nNodes, 1);
for n = 1:nNodes
    % initialize belief to equal uncertainty.
    nodes{n}.marginal = ones(K_states,1) / K_states;
    % This is first to find which are the neighboring nodes for the node of interest 
    links = getRasterNeighbors(n, N_Nodes, M_Nodes);

    %count is introduced here not to make any link to 0
    count = 1;
    for w = 1:4
        if(links(w) ~= 0)
            nodes{n}.links{count} = cell(1, 1);
            nodes{n}.links{count}.farsIndex = links(w);
            % for generality, specify a propMat for each link and direction
            % when w = 1, then the node we are attaching is the left node.
            % When w = 2, then the node we are attaching is the right node.
            % When w = 3, then the node we are attaching is the top node.
            % When w = 4, then the node we are attaching is the bottom node.
            if w == 1
                nodes{n}.links{count}.propMat = cRL;
            elseif w == 2
                nodes{n}.links{count}.propMat = cLR;
            elseif w == 3
                nodes{n}.links{count}.propMat = cDU;
            else
                nodes{n}.links{count}.propMat = cUD;
            end
            count = count + 1;
        end
    end
    
    % Total number of links for that node
    nodes{n}.nLinks = count-1;
end

%------------------------------------------------------------------
% now go through every link, and set the links back and 
% check that each link out has exactly one link back.
%------------------------------------------------------------------

fprintf(1,'Checking bidirectionality ... \n');

for n = 1:nNodes
    
    for li = 1:(nodes{n}.nLinks)
        farIdx = nodes{n}.links{li}.farsIndex;
        nhits = 0;
        for farslink = 1:nodes{farIdx}.nLinks
            if (nodes{farIdx}.links{farslink}.farsIndex == n)
                nodes{n}.links{li}.farsLinkToMe = farslink;
                nhits = nhits + 1;
            end
        end
        if (nhits~= 1)
            keyboard
            error('nhits <> 0 in initNodes');
        end
    end
end

fprintf(1,'Done initting nodes: %f secs\n',toc);
